
' Plane's physics...

Class PlaneBehaviour Extends Behaviour
	
	Field plane_body:RigidBody

	Field roll_torque:Float
	Field pitch_torque:Float

	Field throttle:Float	= 1650000.0
	
	Method New (entity:Entity, mass:Float, roll_rate:Float, pitch_rate:Float)
		
		Super.New (entity)
		
		AddInstance ()

		Local plane_size:Float					= 16.83 * 0.5
		Local plane_box:Boxf					= New Boxf (-plane_size, -plane_size, -plane_size, plane_size, plane_size, plane_size)
	
		' Get a Model-specific handle to the entity:
		
		Local plane_model:Model					= Cast <Model> (entity)
		
		plane_model.Mesh.FitVertices (plane_box)
		plane_model.Mesh.Rotate (0, 180, 0)
		
		plane_model.Move (0.0, 5.0, 0.0)

		' Cull hidden tris...
		
		For Local mat:Material = Eachin plane_model.Materials
			mat.CullMode = CullMode.Back	
		Next

		' Physics setup...

		Local plane_collider:SphereCollider		= entity.AddComponent <SphereCollider> ()

			plane_collider.Radius				= plane_size * 0.6
		
		Local plane_body:RigidBody				= entity.AddComponent <RigidBody> ()

			plane_body.Mass						= mass
			plane_body.Restitution				= 0.5
			plane_body.AngularDamping			= 0.9
			plane_body.LinearDamping			= 0.75
			plane_body.Friction					= 0.0

		roll_torque								= roll_rate
		pitch_torque							= pitch_rate
		
	End
	
	Method OnStart () Override

		' Storing this just for typing convenience...
		
		plane_body = Entity.GetComponent <RigidBody> ()

		' Give it a shove...
		
		plane_body.ApplyImpulse (Entity.Basis * New Vec3f (0.0, 0.0, 500000.0))

	End
	
	Method OnUpdate (elapsed:Float) Override

		' -Scene.GetCurrent ().World.Gravity.Y * 1.0

		plane_body.ApplyForce (Entity.Basis * New Vec3f (0.0, 0.0, throttle))
		
		If Keyboard.KeyDown (Key.A)
			throttle = throttle + 10000.0
		Endif
		
		If Keyboard.KeyDown (Key.Z)
			throttle = throttle - 10000.0
		Endif
		
		If Keyboard.KeyDown (Key.Left)
			plane_body.ApplyTorque (Entity.Basis * New Vec3f (0.0, 0.0, roll_torque))
		Endif

		If Keyboard.KeyDown (Key.Right)
			plane_body.ApplyTorque (Entity.Basis * New Vec3f (0.0, 0.0, -roll_torque))
		Endif

		If Keyboard.KeyDown (Key.Up)
			plane_body.ApplyTorque (Entity.Basis * New Vec3f (pitch_torque, 0.0, 0.0))
		Endif

		If Keyboard.KeyDown (Key.Down)
			plane_body.ApplyTorque (Entity.Basis * New Vec3f (-pitch_torque, 0.0, 0.0))
		Endif
		
	End
	
End
